//
//  CoreDataStack.swift
//  RWPS
//
//  Created by Mic Pringle on 04/12/2014.
//  Copyright (c) 2014 Razeware LLC. All rights reserved.
//

import UIKit
import CoreData

class CoreDataStack {
  
  let context: NSManagedObjectContext
  let persistentStoreCoordinator: NSPersistentStoreCoordinator
  let model: NSManagedObjectModel
  let store: NSPersistentStore?
  
  lazy var storeURL: NSURL = {
    let fileManager = NSFileManager.defaultManager()
    let appGroupIdentifier = "group.com.micpringle.RWPS"
    let containerURL = fileManager.containerURLForSecurityApplicationGroupIdentifier(appGroupIdentifier)
    return containerURL!.URLByAppendingPathComponent("RWPS.sqlite")
  }()
  
  private let databasePopulatedKey = "databasePopulated"
  private lazy var dateFormatter: NSDateFormatter = {
    let formatter = NSDateFormatter()
    formatter.dateFormat = "dd/MM/yy"
    return formatter
  }()
  
  init() {
    let bundle = NSBundle.mainBundle()
    let modelURL = bundle.URLForResource("RWPS", withExtension: "momd")
    model = NSManagedObjectModel(contentsOfURL: modelURL!)!
    persistentStoreCoordinator = NSPersistentStoreCoordinator(managedObjectModel: model)
    context = NSManagedObjectContext()
    context.persistentStoreCoordinator = persistentStoreCoordinator
    let options = [NSMigratePersistentStoresAutomaticallyOption: true]
    var error: NSError? = nil
    store = persistentStoreCoordinator.addPersistentStoreWithType(NSSQLiteStoreType,
      configuration: nil, URL: storeURL, options: options, error: &error)
    if store == nil {
      println("Error adding persistent store: \(error)")
      abort()
    }
  }
  
  // MARK: CoreDataStack
  
  func allParcels() -> [Parcel] {
    let dateSortDescriptor = NSSortDescriptor(key: "deliveryDate", ascending: true)
    let fetchRequest = NSFetchRequest(entityName: "Parcel")
    fetchRequest.sortDescriptors = [dateSortDescriptor]
    if let results = context.executeFetchRequest(fetchRequest, error: nil) as? [Parcel] {
      return results
    }
    return []
  }
  
  func nextParcel() -> Parcel? {
    let dateSortDescriptor = NSSortDescriptor(key: "deliveryDate", ascending: true)
    let fetchRequest = NSFetchRequest(entityName: "Parcel")
    fetchRequest.fetchLimit = 1
    fetchRequest.sortDescriptors = [dateSortDescriptor]
    if let results = context.executeFetchRequest(fetchRequest, error: nil) as? [Parcel] {
      if results.count > 0 {
        return results[0]
      }
    }
    return nil
  }

  func populateDatabaseWithParcels() {
    if !NSUserDefaults().boolForKey(databasePopulatedKey) {
      let bundle = NSBundle.mainBundle()
      let seedDataURL = bundle.URLForResource("Parcels", withExtension: "json")
      let parcelsData = NSData(contentsOfURL: seedDataURL!)
      let parcels = NSJSONSerialization.JSONObjectWithData(parcelsData!, options: NSJSONReadingOptions.AllowFragments, error: nil) as NSArray
      let entity = NSEntityDescription.entityForName("Parcel", inManagedObjectContext: context)
      for i in 0..<parcels.count {
        let dictionary = parcels[i] as NSDictionary
        let parcel = Parcel(entity: entity!, insertIntoManagedObjectContext: context)
        configureParcel(parcel, usingDictionary: dictionary)
      }
      var error: NSError? = nil
      if !context.save(&error) {
        println("Failed to seed database: \(error), \(error?.userInfo)")
        return
      }
      NSUserDefaults().setBool(true, forKey: databasePopulatedKey)
      NSUserDefaults().synchronize()
    }
  }
  
  // MARK: Private
  
  private func configureParcel(parcel: Parcel, usingDictionary dictionary: NSDictionary) {
    parcel.deliveryDate = dateFormatter.dateFromString(dictionary["deliveryDate"] as String)!
    parcel.itemName = dictionary["itemName"] as String
    parcel.location = dictionary["location"] as String
    parcel.latitude = NSNumber(double: dictionary["latitude"] as Double)
    parcel.longitude = NSNumber(double: dictionary["longitude"] as Double)
    parcel.orderDate = dateFormatter.dateFromString(dictionary["orderDate"] as String)!
    parcel.orderNo = dictionary["orderNo"] as String
    parcel.status = dictionary["status"] as String
    parcel.supplierName = dictionary["supplierName"] as String
  }
  
}

